import { Injectable } from '@angular/core';
import { DataStateChangeEventArgs, Sorts, DataResult } from '@syncfusion/ej2-angular-grids'
import { Observable } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { Subject } from 'rxjs';
import { map, tap } from 'rxjs/operators';

@Injectable({
    providedIn: 'root'
  })
export class OrdersService extends Subject<DataStateChangeEventArgs> {
    private BASE_URL = 'https://ej2services.syncfusion.com/production/web-services/api/Orders';

    constructor(private http: HttpClient) {
        super();
    }

    public execute(state: any): void {
        this.getData(state).subscribe(x => super.next(x));
    }

    protected getData(state: DataStateChangeEventArgs): Observable<DataStateChangeEventArgs> {
        const pageQuery = `$skip=${state.skip}&$top=${state.take}`;
        let sortQuery: string = '';

        if ((state.sorted || []).length) {
            sortQuery = `&$orderby=` + state.sorted.map((obj: Sorts) => {
                return obj.direction === 'descending' ? `${obj.name} desc` : obj.name;
            }).reverse().join(',');
        }

        return this.http
           .get(`${this.BASE_URL}?${pageQuery}${sortQuery}&$inlinecount=allpages&$format=json`) 
           .pipe(
            tap((x) => console.log(x)),
            map((response: any) => ({
                result: response.result,
                count: parseInt(response.count, 10)
        }) as DataResult))
        .pipe((data: any) => data);
    }
}
